from .field import Field  # NOQA
from .datetime_field import DateTime  # NOQA
from .date import Date  # NOQA
from .string import String  # NOQA
from .object import Object  # NOQA
from .list import List  # NOQA
from .polymorphic_object import PolymorphicObject  # NOQA
from .polymorphic_list import PolymorphicList  # NOQA
from .constant import Constant  # NOQA
from .int import Int  # NOQA
from .float import Float  # NOQA
from .number import Number  # NOQA
from .bool import Bool  # NOQA
from .dict import Dict  # NOQA



from lupin import Mapper, Schema, fields as f
from models import Artist

mapper = Mapper()
artist_schema = Schema({
    "name": f.String(),
    "birthDate": f.DateTime(binding="birth_date", format="%Y-%m-%d")
}, name="artist")

mapper.register(Artist, artist_schema)

data = {
    "name": "Leonardo da Vinci"
}

artist = mapper.load(data, "artist", allow_partial=True)
assert artist.name == "Leonardo da Vinci"
assert not hasattr(artist, "birth_date")



import pytest

from lupin import fields as f, errors as e, validators as v, Mapper, processors


@pytest.fixture
def mapper():
    return Mapper()


@pytest.fixture
def field():
    return f.Field(validators=[v.Type(str)],
                   pre_dump=[processors.lower],
                   post_dump=[processors.strip],
                   pre_load=[processors.upper])


class TestDump(object):
    def test_returns_the_same_value(self, field, mapper):
        assert field.dump("46", mapper) == "46"


class TestLoad(object):
    def test_returns_the_same_value(self, field, mapper):
        assert field.load("46", mapper) == "46"


class TestPreLoad(object):
    def test_use_processor(self, field):
        assert field.pre_load("hello") == "HELLO"


class TestPostLoad(object):
    def test_use_default_processor(self, field):
        assert field.post_load("hello") == "hello"


class TestExtractAttr(object):
    def test_returns_attr_from_object(self, field, thief, mapper):
        thief.last_name = " lupin  "
        value = field.extract_attr(thief, mapper, "last_name")
        assert value == "lupin"


class TestValidate(object):
    def test_raise_error_if_invalid_data(self, field, mapper):
        with pytest.raises(e.InvalidType):
            field.validate(46, [], mapper)

    def test_do_not_raise_error_if_allow_none_is_trye(self, mapper):
        field = f.Field(validators=[v.Type(str)], allow_none=True)
        field.validate(None, [], mapper)



from lupin.processors import upper


class TestUpper(object):
    def test_does_nothing_if_none(self):
        assert upper(None) is None

    def test_upper_string(self):
        assert upper("HeLLo") == "HELLO"



def constructor(cls, values):
    """Use class constructor to build a new object"""
    return cls(**values)


def bind(cls, values):
    """Create a new instance of class then bind attributes to instance"""
    obj = cls.__new__(cls)
    for attr_name, value in values.items():
        setattr(obj, attr_name, value)
    return obj



import warnings
from functools import reduce
from lupin import ValidatorsAndCombination, ValidatorsNullCombination
from ..validators import IsNone
from ..processors import null_processor


def _make_processor(processors):
    """Returns a callable for executing the processors

    Args:
        processors (list): a list of processor functions to execute
    """
    if not processors:
        return null_processor

    return lambda value: reduce(lambda new_value, proc: proc(new_value), processors, value)


class Field(object):
    """Generic field that does not convert the values"""

    def __init__(self, binding=None, default=None, validators=None, read_only=False, write_only=False,
                 optional=False, allow_none=False, pre_load=None, post_load=None,
                 pre_dump=None, post_dump=None, ignore_if_null=False):
        """
        Args:
            binding (str): attribute name to map on object
            default (object): default value if data is absent
            validators (list|ValidatorsCombination|Validator): list of validators or a combination a validators
            read_only (bool): if True the field will only be used to serialize data
            write_only (bool): if True the field will only be used to load data
            optional (bool): if True it won't raise an error if no value provided for this field
            allow_none (bool): if True None is a accepted has a valid value
            pre_load (list): list of processors to execute before loading the value
            post_load (list): list of processors to execute after loading the value
            pre_dump (list): list of processors to execute before dumping the value
            post_dump (list): list of processors to execute after dumping the value
            ignore_if_null (bool): if True the fields will not be serialize when null
        """
        if validators is None:
            validators = ValidatorsNullCombination()
        elif isinstance(validators, (list, tuple)):
            warnings.warn("List of validators is deprecated, please use combinations (&|)", DeprecationWarning)
            validators = ValidatorsAndCombination(validators)

        if allow_none:
            validators = IsNone() | validators

        self.binding = binding
        self.default = default
        self._validators = validators or []
        self.is_read_only = read_only
        self.is_write_only = write_only
        self.is_optional = optional
        self.is_ignore_if_null = ignore_if_null
        self._pre_load_processor = _make_processor(pre_load)
        self._post_load_processor = _make_processor(post_load)
        self._pre_dump_processor = _make_processor(pre_dump)
        self._post_dump_processor = _make_processor(post_dump)

    def pre_load(self, value):
        return self._pre_load_processor(value)

    def post_load(self, value):
        return self._post_load_processor(value)

    def _pre_dump(self, value):
        return self._pre_dump_processor(value)

    def _post_dump(self, value):
        return self._post_dump_processor(value)

    def load(self, value, mapper):
        """Loads python object from JSON value

        Args:
            value (object): a value
            mapper (Mapper): mapper used to load data

        Returns:
            object
        """
        return value

    def dump(self, value, mapper):
        """Dump value to its JSON representation

        Args:
            value (object): a value
            mapper (Mapper): mapper used to dump data

        Returns:
            object
        """
        return value

    def extract_attr(self, obj, mapper, key=None):
        """Get value of the `key` attribute of object.
        If field has been provided a `binding` then it will
        override `key`

        Args:
            obj (object): object to get value from
            mapper (Mapper): mapper used to dump data
            key (str): attribute name

        Returns:
            object
        """
        key = self.binding or key
        raw_value = getattr(obj, key)
        value = self._pre_dump(raw_value)
        value = self.dump(value, mapper)
        return self._post_dump(value)

    def validate(self, value, path, mapper):
        """Validate value againt field validators

        Args:
            value (object): value to validate
            path (list): JSON path of value
            mapper (Mapper): mapper used to validate data
        """
        self._validators(value, path)



from .validator import Validator  # NOQA
from .type import Type  # NOQA
from .equal import Equal  # NOQA
from .match import Match  # NOQA
from .in_validator import In  # NOQA
from .length import Length  # NOQA
from .datetime_format import DateTimeFormat  # NOQA
from .between import Between  # NOQA
from .url import URL  # NOQA
from .is_none import IsNone  # NOQA
from .typed_dict import TypedDict  # NOQA
from .dict_keys_format import DictKeysFormat  # NOQA



import pytest

from lupin.fields import Float
from lupin.errors import InvalidType
from lupin import Mapper


@pytest.fixture
def mapper():
    return Mapper()


@pytest.fixture
def field():
    return Float()


class TestValidate(object):
    def test_raises_error_if_not_an_float(self, field, mapper):
        with pytest.raises(InvalidType):
            field.validate("46.0", [], mapper)

    def test_does_nothing_if_float(self, field, mapper):
        field.validate(46.0, [], mapper)



from .validator import Validator
from ..errors import InvalidType


class TypedDict(Validator):
    """Validates that keys and values are well typed"""
    def __init__(self, key_type, value_type):
        """
        Args:
            key_type (str): expected type for all keys
            value_type (str): expected type for all values
        """
        self._key_type = key_type
        self._value_type = value_type

    def __call__(self, value, path):
        """Validate items of dict are well typed

        Args:
            value (dict): a dictionnary
            path (list): error path
        """
        if value is None:
            return

        for key, val in value.items():
            if not isinstance(key, self._key_type):
                raise InvalidType(invalid=type(key),
                                  expected=self._key_type,
                                  path=path + [key])

            if not isinstance(val, self._value_type):
                raise InvalidType(invalid=type(key),
                                  expected=self._value_type,
                                  path=path + [key])



from . import Field
from .compatibility import merge_validator
from ..validators import Type
from ..errors import MissingPolymorphicKey, InvalidPolymorphicType


class PolymorphicObject(Field):
    """Field used to handle a key that can contain multiple object types

    Example:
        PolymorphicObject(on="type",
                          schemas={
                              "diamond": diamond_schema,
                              "painting": painting_schema
                          })
    """
    def __init__(self, on, schemas, default_on = None, **kwargs):
        """
        Args:
            on (str): JSON key used to get the object type
            schemas (dict): schemas used for each values used for the `on` key
            default_on (str): default value if the 'on' field is not available
              usefull for update of schema from List to PolymorphicList

        """
        merge_validator(kwargs, Type(dict))
        super(PolymorphicObject, self).__init__(**kwargs)
        self._on = on
        self._default_on = default_on
        self._schemas_by_json_value = schemas
        self._schemas = list(schemas.values())

    def load(self, value, mapper):
        """Loads python objects from JSON object

        Args:
            value (dict): JSON data
            mapper (Mapper): mapper used to load data

        Returns:
            object
        """
        if value is None:
            return None

        schema = self._schemas_by_json_value[value.get(self._on, self._default_on)]
        return mapper.load(value, schema)

    def dump(self, value, mapper):
        """Dump object

        Args:
            value (object): objects to dump
            mapper (Mapper): mapper used to dump data

        Returns:
            dict
        """
        if value is None:
            return value

        mapping = mapper.get_object_mapping(value, self._schemas)
        return mapping.dump(value, mapper)

    def validate(self, value, path, mapper):
        """Validate each items of list against nested mapping.

        Args:
            value (list): value to validate
            path (list): JSON path of value
            mapper (Mapper): mapper used to validate data
        """
        super(PolymorphicObject, self).validate(value, path, mapper)
        if value is not None:
            if self._on not in value:
                raise MissingPolymorphicKey(self._on, path)

            obj_type = value[self._on]
            if obj_type not in self._schemas_by_json_value:
                raise InvalidPolymorphicType(invalid_type=obj_type,
                                             supported_types=list(self._schemas_by_json_value.keys()),
                                             path=path+[self._on])

            schema = self._schemas_by_json_value[obj_type]
            mapping = mapper.get_schema_mapping(schema)
            mapping.validate(value, mapper, path=path)



# coding: utf-8
import pytest

from lupin import bind, constructor
from tests.fixtures import Thief


class TestBind(object):
    def test_bind_custom_attributes_on_instance(self):
        thief = bind(Thief, {"age": 28})
        assert isinstance(thief, Thief)
        assert thief.age == 28


class TestConstructor(object):
    def test_throw_error_if_invalid_argument(self):
        with pytest.raises(TypeError):
            constructor(Thief, {"age": 28})

    def test_returns_a_thief(self):
        arsene = constructor(Thief, {"first_name": "Arsène", "last_name": "Lupin"})
        assert isinstance(arsene, Thief)
        assert arsene.first_name == "Arsène"
        assert arsene.last_name == "Lupin"



from . import Field
from .compatibility import merge_validator
from ..validators import Type
from ..errors import MissingPolymorphicKey, InvalidPolymorphicType


class PolymorphicList(Field):
    """Field used to handle a list containing objects
    of different types, that need to be loaded, serialized
    in different ways.

    Example:
        PolymorphicList(on="type",
                        schemas={
                            "diamond": diamond_schema,
                            "painting": painting_schema
                        })
    """
    def __init__(self, on, schemas, default_on=None, **kwargs):
        """
        Args:
            on (str): JSON key used to get the object type
            schemas (dict): schemas used for each values used for the `on` key
            default_on (str): default value if the 'on' field is not available
              usefull for update of schema from List to PolymorphicList
        """
        merge_validator(kwargs, Type(list))
        super(PolymorphicList, self).__init__(**kwargs)
        self._on = on
        self._default_on = default_on
        self._schemas_by_json_value = schemas
        self._schemas = list(schemas.values())

    def load(self, value, mapper):
        """Loads list of python objects from JSON list

        Args:
            value (list): JSON list
            mapper (Mapper): mapper used to load data

        Returns:
            list
        """
        if value is None:
            return None

        return [mapper.load(item, self._schemas_by_json_value[item.get(self._on, self._default_on)])
                for item
                in value]

    def dump(self, value, mapper):
        """Dump list of object

        Args:
            value (list): objects to dump
            mapper (Mapper): mapper used to dump data

        Returns:
            list
        """
        if value is None:
            return value

        return [mapper.get_object_mapping(item, self._schemas).dump(item, mapper)
                for item in value]

    def validate(self, value, path, mapper):
        """Validate each items of list against nested mapping.

        Args:
            value (list): value to validate
            path (list): JSON path of value
            mapper (Mapper): mapper used to validate data
        """
        super(PolymorphicList, self).validate(value, path, mapper)
        if value is not None:
            for item_index, item in enumerate(value):
                item_path = path + [str(item_index)]
                if self._on not in item:
                    raise MissingPolymorphicKey(self._on, path)

                obj_type = item[self._on]
                if obj_type not in self._schemas_by_json_value:
                    raise InvalidPolymorphicType(invalid_type=obj_type,
                                                 supported_types=list(self._schemas_by_json_value.keys()),
                                                 path=path+[self._on])

                schema = self._schemas_by_json_value[obj_type]
                mapping = mapper.get_schema_mapping(schema)
                mapping.validate(item, mapper, path=item_path)



from lupin.processors import lower


class TestLower(object):
    def test_does_nothing_if_none(self):
        assert lower(None) is None

    def test_lower_string(self):
        assert lower("HeLLo") == "hello"



def null_processor(value):
    """Processor that does nothing, just returns the value"""
    return value


def strip(value):
    """Removes whitespaces from value on both side"""
    return value.strip() if value else value


def lower(value):
    """Lowercase the string passed as argument"""
    return value.lower() if value else value


def upper(value):
    """Uppercase the string passed as argument"""
    return value.upper() if value else value



# -*- coding: utf-8 -*-
import pytest
from lupin import ValidatorsAndCombination, ValidatorsOrCombination
from lupin.validators import Equal
from lupin.errors import ValidationError


@pytest.fixture
def invalid():
    return Equal("sernine")


@pytest.fixture
def valid():
    return Equal("lupin")


class TestCall(object):
    def test_raise_error_if_at_lease_one_validators_is_invalid(self, valid, invalid):
        combination = ValidatorsAndCombination([valid, valid, invalid])
        with pytest.raises(ValidationError):
            combination("andrésy", [])

    def test_do_not_raise_error_if_all_validators_are_valid(self, valid):
        combination = ValidatorsAndCombination([valid, valid, valid])
        combination("lupin", [])


class TestAnd(object):
    def test_returns_self_if_other_is_a_validator(self, valid):
        combination = valid & valid
        assert (combination & valid) is combination

    def test_returns_self_if_other_is_and_combination(self, valid):
        combination = valid & valid
        combination2 = valid & valid
        assert (combination & combination2) is combination

    def test_else_returns_a_new_combination(self, valid):
        combination = valid & valid
        combination2 = valid | valid
        result = combination & combination2
        assert result is not combination
        assert isinstance(result, ValidatorsAndCombination)


class TestOr(object):
    def test_returns_an_or_combination(self, valid):
        combination = valid & valid
        result = combination | valid
        assert isinstance(result, ValidatorsOrCombination)



from lupin.errors import InvalidType
from lupin.validators.typed_dict import TypedDict
import pytest


@pytest.fixture
def validator():
    return TypedDict(str, str)


class TestCall(object):
    def test_do_not_raise_error_if_none(self, validator):
        validator(None, [])

    def test_do_not_raise_error_if_valid_dict(self, validator):
        validator({"a": "a"}, [])

    def test_raise_error_if_invalid_key_type(self, validator):
        with pytest.raises(InvalidType):
            validator({1: "1"}, [])

    def test_raise_error_if_invalid_value_type(self, validator):
        with pytest.raises(InvalidType):
            validator({"1": 1}, [])



from collections import defaultdict

from . import Mapping, bind
from .errors import MissingMapping, SchemaAlreadyRegistered, InvalidPolymorphicType, SchemaNotRegistered


class Mapper(object):
    """Mapper is used to associate the schemas with their python classes"""

    def __init__(self, default_factory=bind):
        """
        Args:
            default_factory (callable): default factory used to instantiate objects
        """
        self._default_factory = default_factory
        self._schemas_to_mapping = {}
        self._schemas_by_name = {}
        self._classes_to_mappings = defaultdict(list)

    def register(self, cls, schema, factory=None):
        """Associate a class with a schema

        Args:
            cls (class): a python class to register with schema
            schema (Schema): schema that will be used to dump & load objects of cls
            factory (callable): factory method used to instantiate objects when loading from JSON
        """
        if schema.name in self._schemas_by_name:
            raise SchemaAlreadyRegistered(schema.name)

        factory = factory or self._default_factory
        mapping = Mapping(cls, schema, factory)
        self._schemas_by_name[schema.name] = schema
        self._schemas_to_mapping[schema] = mapping
        self._classes_to_mappings[cls].append(mapping)

    def validate(self, data, schema, allow_partial=False):
        """Validate data format

        Args:
            data (dict|list): JSON data
            schema (Schema|str): schema (or name) used to validate data
            allow_partial (bool): allow partial schema, won't raise error if missing keys
        """
        if isinstance(data, (list, set, tuple)):
            for item in data:
                self.validate(item, schema, allow_partial)
        else:
            mapping = self.get_schema_mapping(schema)
            mapping.validate(data, self, allow_partial=allow_partial)

    def load(self, data, schema, allow_partial=False):
        """Loads an instance of klass from JSON data

        Args:
            data (dict|list): JSON data
            schema (Schema|str): schema (or name) used to load data
            allow_partial (bool): allow partial schema, won't raise error if missing keys

        returns:
            object
        """
        if isinstance(data, (list, set, tuple)):
            return [self.load(item, schema, allow_partial) for item in data]

        mapping = self.get_schema_mapping(schema)
        return mapping.load(data, self, allow_partial=allow_partial)

    def load_polymorphic(self, data, on, schemas, allow_partial=False):
        """Loads an object that can be of different type
        based on the `on` field

        Args:
            data (dict|list): JSON data
            on (str): JSON key used to get the object type
            schemas (dict): schemas used for each values used for the `on` key
            allow_partial (bool): allow partial schema, won't raise error if missing keys

        Returns:
            list|object
        """
        if isinstance(data, (list, set, tuple)):
            return [self.load_polymorphic(item, on, schemas, allow_partial) for item in data]

        obj_type = data.get(on)
        if obj_type not in schemas:
            raise InvalidPolymorphicType(invalid_type=obj_type,
                                         supported_types=list(schemas.keys()),
                                         path=[on])

        schema = schemas[obj_type]
        return self.load(data, schema, allow_partial)

    def load_attrs(self, data, schema, allow_partial=False):
        """Loads attributes dictionary from `data`

        Args:
            data (dict): dictionary of data
            schema (Schema|str): schema used to load data (or its name)
            allow_partial (bool): allow partial schema, won't raise error if missing keys

        Returns:
            dict
        """
        if isinstance(data, (list, set, tuple)):
            return [self.load_attrs(item, schema, allow_partial) for item in data]

        mapping = self.get_schema_mapping(schema)
        return mapping.load_attrs(data, self, allow_partial=allow_partial)

    def dump(self, obj, schema=None):
        """Dump object into its JSON representation.
        If no schema provided the one used on register() will be used.

        Args:
            obj (object|list): object to dump
            schema (Schema|str): force a schema to dump object (Schema object or a name)

        Returns:
            dict
        """
        if isinstance(obj, (list, set, tuple)):
            return [self.dump(item, schema) for item in obj]

        schemas = (schema,) if schema else None
        mapping = self.get_object_mapping(obj, schemas)

        return mapping.dump(obj, self)

    def get_object_mapping(self, obj, schemas=None):
        """Get mapping of obj.
        If schemas is provided it will only look for mappings associted
        to those schemas.

        Args:
            obj (object): object to get mapping for
            schemas (list<Schema|str>): list of Schema

        Returns:
            Mapping
        """
        try:
            obj_type = obj.__lupin_type__ if hasattr(obj, "__lupin_type__") else type(obj)
            if schemas is not None:
                mappings = [self.get_schema_mapping(schema)
                            for schema in schemas]
                for mapping in mappings:
                    if mapping.can_handle(obj):
                        return mapping

            return self._classes_to_mappings[obj_type][0]
        except KeyError as error:  # one of the schemas was not registered
            schema = error.args[0]
            raise SchemaNotRegistered(schema.name)
        except IndexError:  # No mapping in self._classes_to_mappings
            # try to find a mapping that can handle the parent class of object
            # and register it for future calls
            mappings = self._classes_to_mappings.values()
            for type_mappings in mappings:
                for mapping in type_mappings:
                    if mapping.can_handle(obj):
                        self.register(obj_type, mapping.schema.copy())
                        return self.get_object_mapping(obj)

        raise MissingMapping(obj_type)

    def get_schema_mapping(self, schema):
        """Returns mapping associated to schema.

        Args:
            schema (Schema|str): a schema or its name

        Returns:
            Mapping
        """
        if isinstance(schema, str):
            schema = self._schemas_by_name[schema]
        return self._schemas_to_mapping[schema]



from .validator import Validator
from lupin.errors import InvalidMatch


class DictKeysFormat(Validator):
    """Validates that keys of dict matches a pattern"""
    def __init__(self, pattern):
        """
        Args:
            pattern (SRE_Pattern): expected pattern for dict keys
        """
        self._pattern = pattern

    def __call__(self, value, path):
        """Validate dict keys match the pattern

        Args:
            value (dict): a dictionnary
            path (list): error path
        """
        if value is None:
            return

        for key in value.keys():
            if not self._pattern.match(key):
                raise InvalidMatch(
                    invalid=key,
                    regexp=self._pattern,
                    path=path + [key]
                )



# -*- coding: utf-8 -*-
import pytest
from lupin.validators import Equal
from lupin.errors import ValidationError


@pytest.fixture
def invalid():
    return Equal("sernine")


@pytest.fixture
def valid():
    return Equal("lupin")


class TestAnd(object):
    def test_returns_an_and_combination(self, valid, invalid):
        combination = valid & invalid
        with pytest.raises(ValidationError):
            combination("andrésy", [])


class TestOr(object):
    def test_returns_an_and_combination(self, valid, invalid):
        combination = valid | invalid
        combination("lupin", [])



from . import Validator
from ..errors import NotEqual


class Equal(Validator):
    """Validate that a value equals an expected one"""

    def __init__(self, expected):
        """
        Args:
            expected (object): expected value
        """
        self._expected = expected

    def __call__(self, value, path):
        """Validate that value equals expected one

        Args:
            value (object): value to validate
            path (list): error path
        """
        if value != self._expected:
            raise NotEqual(value, self._expected, path)



from lupin.processors import strip


class TestStrip(object):
    def test_does_nothing_if_none(self):
        assert strip(None) is None

    def test_strip_string(self):
        assert strip(" hello ") == "hello"



import numbers

from . import Field
from .compatibility import merge_validator
from ..validators import Type


class Number(Field):
    """Field used to handle float values"""

    def __init__(self, **kwargs):
        merge_validator(kwargs, Type(numbers.Number))
        super(Number, self).__init__(**kwargs)



import pytest

from lupin import Mapper, Object
from lupin.errors import InvalidDocument
from tests.fixtures import Thief


@pytest.fixture
def mapper():
    return Mapper()


@pytest.fixture(autouse=True)
def mapping(mapper, thief_schema):
    mapper.register(Thief, thief_schema)


@pytest.fixture
def field(thief_schema):
    return Object(thief_schema)


class TestLoad(object):
    def test_returns_a_thief(self, field, thief_data, mapper):
        thief = field.load(thief_data, mapper)
        assert isinstance(thief, Thief)

    def test_returns_none_if_none_value(self, field, mapper):
        assert field.load(None, mapper) is None


class TestDump(object):
    def test_returns_thief_data(self, field, thief, thief_data, mapper):
        data = field.dump(thief, mapper)
        assert data == thief_data

    def test_returns_none_if_value_is_null(self, field, mapper):
        assert field.dump(None, mapper) is None


class TestValidate(object):
    def test_do_not_raise_error_if_none_and_allow_none(self, thief_schema):
        field = Object(thief_schema, allow_none=True)
        field.validate(None, [], mapper)

    def test_does_nothing_if_valid(self, field, thief_data, mapper):
        field.validate(thief_data, [], mapper)

    def test_raises_error_if_invalid_data(self, field, thief_data, mapper):
        thief_data["firstName"] = 46
        with pytest.raises(InvalidDocument):
            field.validate(thief_data, [], mapper)



import pytest

from lupin.errors import InvalidIn
from lupin.validators import In


@pytest.fixture
def validator():
    return In({1, 2, 3})


class TestCall(object):
    def test_raise_error_if_invalid_value(self, validator):
        with pytest.raises(InvalidIn):
            validator(4, [])

    def test_does_nothing_if_valid_value(self, validator):
        validator(1, [])



"""Models used in examples"""


class Artist(object):
    def __init__(self, name, birth_date):
        self.name = name
        self.birth_date = birth_date


class Diamond(object):
    def __init__(self, carat):
        self.carat = carat


class Thief(object):
    def __init__(self, name, stolen_items):
        self.name = name
        self.stolen_items = stolen_items


class Painting(object):
    def __init__(self, name, author):
        self.name = name
        self.author = author



import re
from lupin.errors import InvalidMatch
from lupin.validators.dict_keys_format import DictKeysFormat
import pytest


@pytest.fixture
def validator():
    return DictKeysFormat(re.compile("^\d$"))


class TestCall(object):
    def test_do_not_raise_error_if_none(self, validator):
        validator(None, [])

    def test_do_not_raise_error_if_valid_dict(self, validator):
        validator({"1": "a"}, [])

    def test_raise_error_if_invalid_format(self, validator):
        with pytest.raises(InvalidMatch):
            validator({"a": "1"}, [])
